<?php
namespace Teknisa\PLA\Modules\Cardapio\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Pla00121ImportarTipoPratoController extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    public function __construct($crudService, $parameterBag, $dataSourceManager, $environment, $entityManager, $util) {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->util = $util;
    }
    
    public function importarCardapio(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $result = $this->validaGrupoOperador($row["CDOPERADOR"]);
            
            if(sizeof($result)) {
                $paramFilialMatriz = $result[0]["IDMATRIZFILIAL"];
            } else {
                $paramFilialMatriz = 'M';
            }
            
            $nrseq = array();
            foreach($row['NRSEQCNFMES'] as $nrseqcnfmes) {
                $nrseq[] = str_pad((string)$nrseqcnfmes, 3, "0", STR_PAD_LEFT); 
            }
            
            $nrlista = $this->util->insertLista($nrseq);
            $connection = $this->entityManager->getConnection();
            $procedure = new StoredProcedure($connection, 'EXPORTA_CARDAPIO');
            $procedure->addParam(new Param('P_CDFILIORIG',  Param::PARAM_INPUT, $row['CDFILIAL']));
            $procedure->addParam(new Param('P_CDSERVORIG',  Param::PARAM_INPUT, $row['CDSERVFILI']."_"));
            $procedure->addParam(new Param('P_DTINIORIG',   Param::PARAM_INPUT, $row['DTINIFIM']['start']));
            $procedure->addParam(new Param('P_DTFINORIG',   Param::PARAM_INPUT, $row['DTINIFIM']['end']));
            $procedure->addParam(new Param('P_CDFILIDEST',  Param::PARAM_INPUT, $row['CDFILIDEST']));
            $procedure->addParam(new Param('P_CDSERVDEST',  Param::PARAM_INPUT, $row['CDSERVFILIDEST']."_"));
            $procedure->addParam(new Param('P_DTINIDEST',   Param::PARAM_INPUT, $row['DTINIDEST']));
            $procedure->addParam(new Param('P_DTFINDEST',   Param::PARAM_INPUT, $row['DTFINDEST']));
            $procedure->addParam(new Param('P_CONFIGORIG',  Param::PARAM_INPUT, $nrlista."_"));
            $procedure->addParam(new Param('P_IDCOPCOMENS', Param::PARAM_INPUT, $row['copiarComensais']));
            $procedure->addParam(new Param('P_IDSOBREPOR',  Param::PARAM_INPUT, 'N'));
            $procedure->addParam(new Param('P_CDOPERADOR',  Param::PARAM_INPUT, $row['CDOPERADOR']));
            $procedure->addParam(new Param('P_MODOPROC',    Param::PARAM_INPUT, $paramFilialMatriz));
            $procedure->addParam(new Param('P_NRLOGINTCARD',  Param::PARAM_OUTPUT, null));
            $NrLogIntCard = $procedure->execute();
            
            $this->util->deleteLista($nrlista);
            
            $pratosInativos = $this->getPratosReceitasInativas($row);
            if(!empty($pratosInativos)) {
                $response->addDataSet(new DataSet("errorMsg", array("Pratos inativos ou com receita inativa do cardápio de origem não foram importados.")));
                throw new \Exception("Pratos inativos ou com receita inativa do cardápio de origem não foram importados.");
            }
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }
    
    public function validaGrupoOperador($operador) {
        $query = "SELECT 
                    OG.CDGRUPOPER,
                    AG.CDGRUPOPER,
                    CASE 
                        WHEN NOT EXISTS (SELECT CDGRUPOPER FROM AUTGRUPOP WHERE IDAUTGRUPOP = 'LRCD') 
                            THEN 'M'
                        WHEN (EXISTS (SELECT CDGRUPOPER FROM AUTGRUPOP WHERE IDAUTGRUPOP = 'LRCD') AND EXISTS (SELECT A.CDGRUPOPER FROM OPERGRUPOP O JOIN AUTGRUPOP A ON O.CDGRUPOPER = A.CDGRUPOPER WHERE A.IDAUTGRUPOP = 'LRCD' AND CDOPERADOR = :CDOPERADOR)) 
                            THEN 'M' 
                        ELSE 'F'
                    END AS IDMATRIZFILIAL
                FROM (SELECT CDGRUPOPER FROM OPERGRUPOP WHERE CDOPERADOR = :CDOPERADOR) OG
                    FULL OUTER JOIN (SELECT CDGRUPOPER FROM AUTGRUPOP WHERE IDAUTGRUPOP = 'LRCD') AG ON OG.CDGRUPOPER = AG.CDGRUPOPER";
        $params = array(
            "CDOPERADOR" => $operador
        );
        return $this->entityManager->getConnection()->fetchAll($query, $params);
    }
    
    private function getPratosReceitasInativas($row) {
        $query = "SELECT
                    P.CDPRATO,
                    P.CDFILAUXPRAT,
                    P.NMPRATO,
                    P.NRORG,
                    P.IDATIVO AS IDATIVOPRATO,
                    PC.IDATIVO AS IDATIVORECE
                FROM
                    CARDADIA CA,
                    COMPCARD CC,
                    PRATO P,
                    (
                        SELECT
                            CDPRATO,
                            CDFILAUXPRAT,
                            NRCORPRIPRAT AS NRCOR,
                            'P' AS TIPO,
                            IDATIVO
                        FROM
                            PRATPADR
                        UNION ALL
                        SELECT
                            CDPRATO,
                            CDFILAUXPRAT,
                            NRCORPRIFILI AS NRCOR,
                            'F' AS TIPO,
                            IDATIVO
                        FROM
                            PRATFILI
                        WHERE
                            CDFILIAL =:CDFILIAL
                        UNION ALL
                        SELECT
                            CDPRATO,
                            CDFILAUXPRAT,
                            NRCORPRISERV AS NRCOR,
                            'S' AS TIPO,
                            IDATIVO
                        FROM
                            PRATSERV
                        WHERE
                                CDFILIAL =:CDFILIAL
                            AND
                                CDSERVFILI =:CDSERVFILI
                    ) PC
                WHERE
                        CC.NRCARDAPIO = CA.NRCARDAPIO
                    AND
                        CC.CDPRATO = P.CDPRATO
                    AND
                        P.CDPRATO = PC.CDPRATO
                    AND
                        CA.CDSERVFILI = :CDSERVFILI
                    AND
                        CA.CDFILIAL = :CDFILIAL
                    AND
                        CA.DTCARDAPIO BETWEEN :DTCARDAPIOINI AND :DTCARDAPIOFIN
                    AND (
                            P.IDATIVO <> 'S'
                        OR
                            PC.IDATIVO <> 'S'
                    )
                GROUP BY
                    P.CDPRATO,
                    P.CDFILAUXPRAT,
                    P.NMPRATO,
                    P.NRORG,
                    P.IDATIVO,
                    PC.IDATIVO";
        $params = array(
            "CDFILIAL" => $row['CDFILIAL'],
            "CDSERVFILI" => $row['CDSERVFILI'],
            "DTCARDAPIOINI" => $row['DTINIFIM']['start'],
            "DTCARDAPIOFIN" => $row['DTINIFIM']['end']
        );
        return $this->entityManager->getConnection()->fetchAll($query, $params);
    }
}