<?php
namespace Teknisa\PLA\Modules\Prato\Controllers;

use Zeedhi\Framework\DTO;

class Pla13000GrupoPratoReport extends \Zeedhi\Framework\Controller\Report{


    protected $reportService;
    protected $qr2Strategy;
    protected $birtStrategy;
    protected $util;
    protected $crudService;
    protected $environment;
    protected $nrlista;

    public function __construct($reportService, $qr2Strategy, $birtStrategy, $environment, $util, $crudService) {
        $this->reportService = $reportService;
        $this->birtStrategy = $birtStrategy;
        $this->environment = $environment;
        $this->qr2Strategy = $qr2Strategy;
        $this->util = $util;
        $this->crudService = $crudService;
    }

    protected $reportMapping = array (
        'GRP00001' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'CDPRATOINI' => 'P_CDPRATOINICIAL',
                        'CDPRATOFIN' => 'P_CDPRATOFINAL',
                        'NRLISTA'    => 'P_NRLISTA',
                        'IDTIPORESTRICAO' => 'P_IDTIPORESTRICAO',
                        'NRORG' => 'P_NRORG',
                        'CDPICTPRAT' => 'P_CDPICTPRAT'
                    )
            ),
        'GRP00002' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'CDPRATOINI' => 'P_CDPRATOINICIAL',
                        'CDPRATOFIN' => 'P_CDPRATOFINAL',
                        'NRLISTA'    => 'P_NRLISTA',
                        'IDTIPORESTRICAO' => 'P_IDTIPORESTRICAO',
                        'NRORG' => 'P_NRORG',
                        'CDPICTPRAT' => 'P_CDPICTPRAT'
                    )
            ),
        'GRP00003' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'CDPRATOINI' => 'P_CDPRATOINICIAL',
                        'CDPRATOFIN' => 'P_CDPRATOFINAL',
                        'NRLISTA'    => 'P_NRLISTA',
                        'IDTIPORESTRICAO' => 'P_IDTIPORESTRICAO',
                        'DTINICIAL' => 'P_DTINICIAL',
                        'DTFINAL' => 'P_DTFINAL',
                        'NRORG' => 'P_NRORG',
                        'CDPICTPRAT' => 'P_CDPICTPRAT'
                    )
            )
    );

    public function createRemoteReport(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $this->beforeProcessReport($request, $response);
            $reportURL = $this->processReport($request);
            $response->addMessage(new DTO\Response\Message($reportURL));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $cdfiliserv = $this->getCdfiliserv($row);
        $row["NRORG"] = $this->environment->getNrOrg();
        if ((isset($row['CDREPORT'])) && ($row['CDREPORT'] === 'GRP00003')){
            $row["DTINICIAL"] = $row["DTINTERVAL"]["formattedStart"];
            $row["DTFINAL"] = $row["DTINTERVAL"]["formattedEnd"];            
        } 
        $this->nrlista = $this->util->insertLista($cdfiliserv);
        
        $row["NRLISTA"] = $this->nrlista;
        
        if (isset($row['QR2TYPE']) && $row['QR2TYPE']){
          $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
        }
    }

    private function getCdfiliserv($row) {
        if ($row["NMSERVICO_ALL"] != "T") {
            return $row["CDFILISERV"];
        } else {
            $conditions = array(
                array(
                    "columnName" => "CDFILIAL",
                    "operator"   => '=',
                    "value"      => $row["NMFILIAL_ALL"] == "T" ? "T" : implode("_", $row["CDFILIAL"])
                ),
                array(
                    "columnName" => "CDGRUPSERV",
                    "operator"   => '=',
                    "value"      => "T"
                ),
                array(
                    "columnName" => "CDOPERADOR",
                    "operator"   => '=',
                    "value"      => "OPERADOR_LOGADO"
                ),
                array(
                    "columnName" => "CDGRUPFILI",
                    "operator"   => '=',
                    "value"      => isset($row["CDGRUPFILI"]) && !empty($row["CDGRUPFILI"]) ? $row['CDGRUPFILI'] : "T"
                ),
                array(
                    "columnName" => "IDCNTRRESC",
                    "operator"   => '<>',
                    "value"      => 'S'
                )
            );
            $dataSet = $this->crudService->getFilteredDataSet("servico_operador_filgrupfili_filial_grupserv", $conditions);
            $rows = $dataSet->getRows();
            $cdfiliserv = array();
            foreach ($rows as $row) {
                array_push($cdfiliserv, $row["CHAVE"]);
            }
            return $cdfiliserv;
        }
    }
} 
