<?php
namespace Teknisa\PLA\Modules\Relatorio\Controllers;

use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;

class PLA11000NutrientesDoCardapioVetController extends \Zeedhi\Framework\Controller\Report {

    protected $novoCodigo;
    protected $procedureNutriente;
    protected $parameterBagColumns;
    protected $entityManager;
    protected $qr2Strategy;
    protected $birtStrategy;
    protected $nrsequnutr;
    protected $cdfilial;
    protected $nrorg;
    protected $connection;
    protected $environment;
    protected $crudService;
    protected $dataSourceManager;

    public function __construct($reportService, $novoCodigo, $entityManager, $qr2Strategy, $birtStrategy,$environment, $connection,$crudService,$dataSourceManager) {
        
        $this->reportService      = $reportService;
        $this->novoCodigo         = $novoCodigo;
        $this->entityManager      = $entityManager;
        $this->qr2Strategy        = $qr2Strategy;
        $this->birtStrategy       = $birtStrategy;
        $this->environment        = $environment;
        $this->connection         = $connection;
        $this->crudService        = $crudService;
        $this->dataSourceManager  = $dataSourceManager;
    }

    protected $reportMapping = array (
    	'PLA11000' => 
        array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
            array (

                'NRLISTA'     => 'P_NRLISTA',
                'DTINICIAL'      => 'P_DTINI',
                'DTFINAL'      => 'P_DTFIN',
                'NRORG'     =>'P_NRORG'
            ),
        ),
		'PLA11001' => 
        array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
            array (

                'NRLISTA'     => 'P_NRLISTA',
                'DTINICIAL'      => 'P_DTINI',
                'DTFINAL'      => 'P_DTFIN',
                 'NRORG'     =>'P_NRORG'
            ),
        ),
		'PLA11002' => 
        array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
            array (

                'NRLISTA'     => 'P_NRLISTA',
                'DTINICIAL'      => 'P_DTINI',
                'DTFINAL'      => 'P_DTFIN',
                  'NRORG'     =>'P_NRORG'
            ),
        )
    );

    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
       
        $row = $request->getRow();
        $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
        $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
        $this->stNomeRel = $row['REPORT_NAME'];
        $row['__report_name']= $this->stNomeRel;
        $this->getFiliais($request, $response);
        //cria lista de FILIAL concatenada com NRSEQUNUTR
        $nrlista = $this->novoCodigo->getCodeByTableName('LISTA', $this->environment->getNrorgTrab(), 1, 10, false);
       
        
        for ($i=0;$i<count($row['CDFILIAL']);$i++){
			$this->cdfilial = $row['CDFILIAL'][$i];
            $row['L_SERVICO']=$this->retornaServicos($row);
            $row['L_NUTRI'] = $this->getNutrientes();
            $row['NRSEQUNUTR'] = $this->novoCodigo->getCodeByTableName('CALCNUTR'.$row['CDFILIAL'][$i], $row['NRORG']);
            $row['IDNDPCAL']='N';        

            //adiciona FILIAL concatenada com NRSEQUNUTR na lista 
            $this->insereLista($nrlista,$row['CDFILIAL'][$i].$row['NRSEQUNUTR']);
            $procedureNutriente = new StoredProcedure($this->connection, 'NUTRIENTE_DO_CARDAPIO_PLA');
            
            $procedureNutriente->addParam(new Param('P_FILIAL',     Param::PARAM_INPUT, $row['CDFILIAL'][$i],     Param::PARAM_TYPE_STR));
            $procedureNutriente->addParam(new Param('P_SERVICOS',   Param::PARAM_INPUT, $row['L_SERVICO'],   Param::PARAM_TYPE_STR));
            $procedureNutriente->addParam(new Param('P_NUTRIENTES', Param::PARAM_INPUT, $row['L_NUTRI'],     Param::PARAM_TYPE_STR));
            $procedureNutriente->addParam(new Param('P_NRSEQUNUTR', Param::PARAM_INPUT, $row['NRSEQUNUTR'],   Param::PARAM_TYPE_STR));
            $procedureNutriente->addParam(new Param('P_IDNDPCAL',   Param::PARAM_INPUT, $row['IDNDPCAL'], Param::PARAM_TYPE_STR));
            $procedureNutriente->addParam(new Param('P_DTINICIAL',  Param::PARAM_INPUT, $row['DTINICIAL'],      Param::PARAM_TYPE_STR));
            $procedureNutriente->addParam(new Param('P_DTFINAL',    Param::PARAM_INPUT, $row['DTFINAL'],      Param::PARAM_TYPE_STR));
            $procedureNutriente->execute();
        }
        $row['NRLISTA']= $nrlista ;
    }
	protected function getFiliais($request, $response){
		$row = $request->getRow();
		if((is_array($row['CDFILIAL'])!=true) && $row['CDFILIAL']=="T"){
			if(!empty($row['CDGRUPFILI'])){
				$conditions= array(
					array(
						'columnName' => 'CDGRUPFILI',
						'operator' => '=',
						'value' => $row['CDGRUPFILI']
					),
					array(
						"columnName" =>'NRORG',
						"operator" => "=",
						"value" => $this->environment->getNrOrgTrab()
					),
					array(
						"columnName" =>'IDCNTRRESC',
						"operator" => "=",
						"value" => 'N'
					)
				);
				
				
				$this->parameterBagColumns = array("CDGRUPFILI", "NRORG");
				$filterCriteria = new FilterCriteria('filial_filgrupfili', $conditions);
				$newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
				$dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
			} else{
				$conditions=array();
				$filterCriteria = new FilterCriteria('filial', $conditions);
            	$dataSet = $this->dataSourceManager->findBy($filterCriteria);
			}
			$row['CDFILIAL']=array();
			foreach ($dataSet->getRows() as $filial) {
				array_push($row['CDFILIAL'], $filial['CDFILIAL']);
			}
		}
	}
	public function retornaServicos($row) {
        $sql = <<<SQL
           SELECT (SELECT rtrim(xmlagg(xmlelement(e,S.CDSERVFILI,'_').extract('//text()') order by S.CDSERVFILI ).GetClobVal(),'_')
            FROM SERVICO S
            INNER JOIN (
                SELECT
                    F.CDFILIAL, F.NMFILIAL, F.NRORG
                FROM
                    FILIAL        F
                    INNER JOIN FILIOPER FO ON FO.CDFILIAL = F.CDFILIAL
                                          AND FO.NRORG = F.NRORG
                                          AND FO.NRORG = :NRORG
                                          AND FO.CDOPERADOR = :CDOPERADOR
                    INNER JOIN FILGRUPFILI FG ON F.CDFILIAL = FG.CDFILIAL
                                         AND F.NRORG = FG.NRORG
                                         AND FO.NRORG = :NRORG
                                         AND FG.CDGRUPFILI = :CDGRUPFILI
                WHERE ( INSTR(:CDFILIAL, F.CDFILIAL) <> 0 OR :CDFILIAL = 'T' ) 
                      AND IDCNTRRESC = 'N'
                UNION
                SELECT
                    F.CDFILIAL, F.NMFILIAL, F.NRORG
                FROM
                    FILIAL F
                    INNER JOIN FILIOPER FO ON FO.CDFILIAL = F.CDFILIAL
                                          AND FO.NRORG = F.NRORG
                                          AND FO.NRORG = :NRORG
                                          AND FO.CDOPERADOR = :CDOPERADOR
                WHERE ( :CDGRUPFILI = 'T' )
                    AND ( INSTR(:CDFILIAL, F.CDFILIAL) <> 0
                          OR :CDFILIAL = 'T' ) 
                    AND IDCNTRRESC = 'N'
            ) F ON F.CDFILIAL = S.CDFILIAL
                   AND F.NRORG = S.NRORG
                   AND S.IDSERVATIV = 'S'
            WHERE S.CDFILIAL = :CDFILIAL
            AND S.NRORG = :NRORG
        ) AS CDLSERVFILI
            FROM DUAL
SQL;
        $params = array(
            'NRORG'        => $this->environment->getNrOrg(),
            'CDFILIAL'     => $this->cdfilial,
            'CDOPERADOR'   => $this->environment->getCdOperador(),
            'CDGRUPFILI'   => 'T'
        );
        
        try {
            $lista = $this->entityManager->getConnection()->fetchAll($sql, $params);
            return $lista[0]['CDLSERVFILI'];
        } catch (\Exception $e) {
            throw $e;
        }
    }
	public function getNutrientes(){
	    $sql = "SELECT CDCALORIAS, CDPROTEINA, CDCARBOHIDRA, CDALCOOL, CDFIBRAS, CDSODIO, CDLIPIDIOS, CDGORDURASATURADA FROM PARAMGERAL";
        $nutrientes = $this->entityManager->getConnection()->fetchAll($sql);
        $nutrientes =  implode('_', $nutrientes[0]);
        return $nutrientes.'_';
	}
    private function insereLista($nrlista, $valor) {
		$row = array();

			array_push($row,
				array(
					"NRLISTA" 	=> $nrlista,
					"CDLISTA" 	=> $valor,
					"__is_new"  => true
				)
			);

		$dataSet = new DataSet(null, $row);
		$request = new DTO\Request\DataSet($dataSet, '', '', '');
		$rowsToSave = $this->crudService->getRowsToSave($request, 'lista');
		$this->crudService->save('lista', $rowsToSave);
	}
    
}